import java.io.*;
import javax.microedition.io.*;
import com.nttdocomo.io.*;
import com.nttdocomo.ui.*;
import com.nttdocomo.util.*;
import com.nttdocomo.system.*;
import net.sf.jazzlib.*;

public final class GBCanvas extends Canvas {

    public static final int REQUEST_RESET = 1;
    public static final int REQUEST_STOP = 2;
    public static final int REQUEST_LOAD = 3;
    public static final int REQUEST_SAVE = 4;

    private final GBCore gb;

    private final int[] fb;
    private final int[] palette;
    private int frame_skip, frame_count, last_frame;
    private long last_time;
    private String messageStr;
    private boolean zoom, lastzoom = true;

    GBCanvas() {
	setSoftLabel( Frame.SOFT_KEY_1 , "ROM" );
	setSoftLabel( Frame.SOFT_KEY_2 , "RESET" );
	SDbinding sd = new SDbinding();
	sd.createFile("001.GB");
	gb = new GBCore(this);
	fb = new int[240 * 216];
	palette = new int[0x40];
	frame_skip = 5;

	gb.init();
    }

    public void setPixels(int scanline, byte[] b, int margin) {
	final int[] fb = this.fb;
	final int[] palette = this.palette;

	if (zoom) {
	    int j = (scanline >> 1) * 720;
	    if ((scanline & 1) == 0)
		for (int i = 0; i < 160; i += 2) {
		    final int c1 = palette[b[margin + i] & 0x3F];
		    final int c2 = palette[b[margin + i + 1] & 0x3F];
		    fb[j] = c1;
		    fb[j + 1] = (c1 + c2) >> 1;
		    fb[j + 2] = c2;
		    j += 3;
		}
	    else 
		for (int i = 0; i < 160; i += 2) {
		    final int c1 = palette[b[margin + i] & 0x3F];
		    final int c2 = palette[b[margin + i + 1] & 0x3F];
		    fb[j + 480] = c1;
		    fb[j + 481] = (c1 + c2) >> 1;
		    fb[j + 482] = c2;
		    fb[j + 240] = (fb[j] + c1) >> 1;
		    fb[j + 241] = (fb[j] + c2) >> 1;
		    fb[j + 242] = (fb[j + 2] + c2) >> 1;
		    j += 3;
		}
	}
	else {
	    final int j = scanline * 160;
	    for (int i = 0; i < 160; i++)
		fb[i + j] = palette[b[margin + i] & 0x3F];
	}
    }

    public boolean startFrame() {
	return frame_count++ % frame_skip == 0;
    }

    public void endFrame(boolean render_frame) {
	if (!render_frame)
	    return;

	if (frame_count - last_frame >= 60) {
	    long cur_time = System.currentTimeMillis();
	    int fps = 60000 / (int)(cur_time - last_time);
	    last_frame += 60;
	    last_time = cur_time;
	    messageStr = fps + "fps fs=" + frame_skip;
	}

	repaint();
    }

    public int readJoyPad() {
	int key = getKeypadState();
	return
	    ((((key >> Display.KEY_3) | (key >> Display.KEY_SELECT)) & 1) << GBCore.PAD_A) |
	    (((key >> Display.KEY_2    ) & 1) << GBCore.PAD_B) |
	    (((key >> Display.KEY_5    ) & 1) << GBCore.PAD_SELECT) |
	    (((key >> Display.KEY_6    ) & 1) << GBCore.PAD_START) |
	    (((key >> Display.KEY_UP   ) & 1) << GBCore.PAD_UP) |
	    (((key >> Display.KEY_DOWN ) & 1) << GBCore.PAD_DOWN) |
	    (((key >> Display.KEY_LEFT ) & 1) << GBCore.PAD_LEFT) |
	    (((key >> Display.KEY_RIGHT) & 1) << GBCore.PAD_RIGHT);
    }

    public void resetJoyPad() {
    }
    
    public void setPalette(int n, int r, int g, int b) {
	palette[n] = Graphics.getColorOfRGB(r & 0xFE, g & 0xFE, b & 0xFE);
    }

    public void setVibration(boolean vibration) {
	PhoneSystem.setAttribute(PhoneSystem.DEV_VIBRATOR, vibration ? PhoneSystem.ATTR_VIBRATOR_ON : PhoneSystem.ATTR_VIBRATOR_OFF);
    }

    public DataOutputStream getDataOutputStream(int slot) throws IOException { // mini͒
	return Connector.openDataOutputStream("scratchpad:///0;pos="+(slot-1)*100000+",length=100000");
    }

    public DataInputStream getDataInputStream(int slot) throws IOException {
	return Connector.openDataInputStream("scratchpad:///0;pos="+(slot-1)*100000+",length=100000");
    }

    public void setMessage(String s) {
	messageStr = s;
	repaint();
    }


    public void processEvent(int type, int param) {
	if (type == Display.KEY_PRESSED_EVENT) {
	    switch (param) {
	    case Display.KEY_SOFT1:
			try {
			String fnm = "001.GB";
		    SDbinding sd = new SDbinding();
		    InputStream is = sd.openInputStream(fnm);
		    //mini͂
	/*
		    is.mark(2);
		    int b1 = is.read(), b2 = is.read();
		    is.reset();
		    if (b1 == 0x1f && b2 == 0x8b)
		    is = new GZIPInputStream(is);
		    //܂łRgAEgB
	*/
		    int c; 
		    if ((c = gb.loadROM(is)) == 0) {
				gb.reset();
				//nes.run();
				gb.start();
						
				sd.close();
				is = null ;
				sd = null;
		    } else {
				alert("error code=" + c);
		    }
			
		} catch (Exception e) {
		    System.out.println("ROM load error: " + e);
		}
		break;

	    case Display.KEY_SOFT2:
		if (new Dialog(Dialog.DIALOG_YESNO, "Zbg܂H").show() == Dialog.BUTTON_YES)
		    gb.request(REQUEST_RESET, 0);
		break;

	    case Display.KEY_1:
		frame_skip++;
		break;

	    case Display.KEY_4:
		if (frame_skip > 1)
		    frame_skip--;
		break;

	    case Display.KEY_7:
		if (confirm("1ɃZ[u܂H"))
		    gb.request(REQUEST_SAVE, 1);
		break;

	    case Display.KEY_ASTERISK:
		if (confirm("1烍[h܂H"))
		    gb.request(REQUEST_LOAD, 1);
		break;

	    case Display.KEY_8:
		if (confirm("2ɃZ[u܂H"))
		    gb.request(REQUEST_SAVE, 2);
		break;

	    case Display.KEY_0:
		if (confirm("2烍[h܂H"))
		    gb.request(REQUEST_LOAD, 2);
		break;

		/*
	    case Display.KEY_9:
		if (confirm("3ɃZ[u܂H"))
		    gb.request(REQUEST_SAVE, 3);
		break;

	    case Display.KEY_POUND:
		if (confirm("3烍[h܂H"))
		    gb.request(REQUEST_LOAD, 3);
		break;
		*/
	    case Display.KEY_9:
		zoom = !zoom;
		repaint();
		break;
	    case Display.KEY_POUND:
		gb.xxx();
		break;
	    }
	}
    }

    public void paint(Graphics g) {
	g.lock();

	if (zoom != lastzoom) {
	    g.setColor(Graphics.getColorOfName(Graphics.BLACK));
	    g.fillRect(0, 0, 240, 240);
	    lastzoom = zoom;
	}

	if (zoom)
	    g.setPixels((getWidth() - 240) / 2, 0, 240, 216, fb, 0);
	else
	    g.setPixels((getWidth() - 160) / 2, (getHeight() - 144 - 32) / 2, 160, 144, fb, 0);
	
	if (messageStr != null) {
	    g.setColor(Graphics.getColorOfName(Graphics.BLACK));
	    g.fillRect(0, 224, 240, 16);
	    
	    g.setColor(Graphics.getColorOfName(Graphics.WHITE));
	    g.drawString(messageStr, 0, 239);

	    messageStr = null;
	}

	g.unlock(true);
    }

    private void alert(String s) {
	new Dialog(Dialog.DIALOG_ERROR, s).show();
    }

    private boolean confirm(String s) {
	return new Dialog(Dialog.DIALOG_YESNO, s).show() == Dialog.BUTTON_YES;
    }
}
